--[[----------------------------------------------------------------------------

WFCatVideoTimeTest.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local WFCatVideoTime = require 'WFCatVideoTime'

--==============================================================================

local WFCatVideoTimeTest = {}

--------------------------------------------------------------------------------

local checkEqual = function( a, b, msg )
	assert( a == b, msg )
end

--------------------------------------------------------------------------------

WFCatVideoTimeTest.testSMPTE_exhaustive = function()
	
	local kNTSCFrameRate = WFCatVideoTime.kNTSCFrameRate
	
	local success = true
	local f = kNTSCFrameRate[ 1 ] / kNTSCFrameRate[ 2 ]
		
	local function t( n, s, l )
		local t = n / f
		
		local result = WFCatVideoTime.formatSmpteVideoTime( t, kNTSCFrameRate )
		if result ~= s then
			checkEqual( result, s, string.format( "%03.4f %s", t, l or s ) )
			success = false
		end
	end
	
	local tests = {
		{ 0, '00;00;00', 'A0 0 0', },
		{ 250, '00;08;10', 'A0 8 10', },
		{ 251, '00;08;11', 'A0 8 11', },
		{ 1016, '00;33;26', 'A0 33 26', },
		{ 1293, '00;43;03', 'A0 43 3', },
		{ 1790, '00;59;20', 'A0 59 20', },
		{ 2486, '01;22;28', 'C 0 1 22 28', },
		{ 2621, '01;27;13', 'C 0 1 27 13', },
		{ 3182, '01;46;04', 'C 0 1 46 4', },
		{ 4036, '02;14;20', 'C 0 2 14 20', },
		{ 4104, '02;16;28', 'C 0 2 16 28', },
		{ 5473, '03;02;19', 'C 0 3 2 19', },
		{ 6015, '03;20;21', 'C 0 3 20 21', },
		{ 6424, '03;34;10', 'C 0 3 34 10', },
		{ 6523, '03;37;19', 'C 0 3 37 19', },
		{ 6894, '03;50;00', 'C 0 3 50 0', },
		{ 7375, '04;06;03', 'C 0 4 6 3', },
		{ 7657, '04;15;15', 'C 0 4 15 15', },
		{ 7997, '04;26;25', 'C 0 4 26 25', },
		{ 8057, '04;28;25', 'C 0 4 28 25', },
		{ 10535, '05;51;15', 'C 0 5 51 15', },
		{ 10870, '06;02;22', 'C 0 6 2 22', },
		{ 13633, '07;34;27', 'C 0 7 34 27', },
		{ 14049, '07;48;23', 'C 0 7 48 23', },
		{ 14436, '08;01;22', 'C 0 8 1 22', },
		{ 15174, '08;26;10', 'C 0 8 26 10', },
		{ 15307, '08;30;23', 'C 0 8 30 23', },
		{ 15327, '08;31;13', 'C 0 8 31 13', },
		{ 15610, '08;40;26', 'C 0 8 40 26', },
		{ 15621, '08;41;07', 'C 0 8 41 7', },
		{ 15973, '08;52;29', 'C 0 8 52 29', },
		{ 16035, '08;55;01', 'C 0 8 55 1', },
		{ 19532, '10;51;20', 'A1 51 20', },
		{ 21976, '12;13;08', 'C 1 2 13 8', },
		{ 22316, '12;24;18', 'C 1 2 24 18', },
		{ 22607, '12;34;09', 'C 1 2 34 9', },
		{ 22773, '12;39;25', 'C 1 2 39 25', },
		{ 23856, '13;16;00', 'C 1 3 16 0', },
		{ 26491, '14;43;27', 'C 1 4 43 27', },
		{ 26527, '14;45;03', 'C 1 4 45 3', },
		{ 26664, '14;49;20', 'C 1 4 49 20', },
		{ 27224, '15;08;12', 'C 1 5 8 12', },
		{ 28412, '15;48;00', 'C 1 5 48 0', },
		{ 30375, '16;53;15', 'C 1 6 53 15', },
		{ 31027, '17;15;09', 'C 1 7 15 9', },
		{ 31190, '17;20;22', 'C 1 7 20 22', },
		{ 31526, '17;31;28', 'C 1 7 31 28', },
		{ 32395, '18;00;29', 'B 1 8 29', },
		{ 33334, '18;32;08', 'C 1 8 32 8', },
		{ 33630, '18;42;04', 'C 1 8 42 4', },
		{ 33732, '18;45;16', 'C 1 8 45 16', },
		{ 35615, '19;48;11', 'C 1 9 48 11', },
		{ 35781, '19;53;27', 'C 1 9 53 27', },
		{ 35948, '19;59;14', 'C 1 9 59 14', },
		{ 36949, '20;32;25', 'A2 32 25', },
		{ 37701, '20;57;27', 'A2 57 27', },
		{ 39162, '21;46;20', 'C 2 1 46 20', },
		{ 39623, '22;02;03', 'C 2 2 2 3', },
		{ 40083, '22;17;13', 'C 2 2 17 13', },
		{ 40756, '22;39;26', 'C 2 2 39 26', },
		{ 41061, '22;50;01', 'C 2 2 50 1', },
		{ 41125, '22;52;05', 'C 2 2 52 5', },
		{ 42121, '23;25;13', 'C 2 3 25 13', },
		{ 42679, '23;44;01', 'C 2 3 44 1', },
		{ 44515, '24;45;09', 'C 2 4 45 9', },
		{ 44713, '24;51;27', 'C 2 4 51 27', },
		{ 44871, '24;57;05', 'C 2 4 57 5', },
		{ 44968, '25;00;14', 'B 2 5 14', },
		{ 45081, '25;04;07', 'C 2 5 4 7', },
		{ 46212, '25;41;28', 'C 2 5 41 28', },
		{ 46264, '25;43;20', 'C 2 5 43 20', },
		{ 46622, '25;55;18', 'C 2 5 55 18', },
		{ 47069, '26;10;17', 'C 2 6 10 17', },
		{ 47673, '26;30;21', 'C 2 6 30 21', },
		{ 49923, '27;45;23', 'C 2 7 45 23', },
		{ 50589, '28;08;01', 'C 2 8 8 1', },
		{ 51634, '28;42;26', 'C 2 8 42 26', },
		{ 52281, '29;04;15', 'C 2 9 4 15', },
		{ 52541, '29;13;05', 'C 2 9 13 5', },
		{ 53335, '29;39;19', 'C 2 9 39 19', },
		{ 53571, '29;47;15', 'C 2 9 47 15', },
	}
	
	for _, test in ipairs( tests ) do
		t( unpack( test ) )
	end

	checkEqual( success, true )
	
end

--------------------------------------------------------------------------------

WFCatVideoTimeTest.testDecode_exhaustive = function()
	
	local kNTSCFrameRate = WFCatVideoTime.kNTSCFrameRate
	
	local fr = kNTSCFrameRate[ 1 ] / kNTSCFrameRate[ 2 ]
	local success = true
	for i = 1049, 30 * 60 * 20, 1301 do
		local t = i / fr

		local formatted = WFCatVideoTime.formatSmpteVideoTime( t, kNTSCFrameRate )
		local decoded = WFCatVideoTime.decodeUserVideoTime( formatted, kNTSCFrameRate, "smpte" )
		if math.abs( t - decoded ) > 0.00000001 then
			success = false
			checkEqual( t, decoded, "decode " .. tostring( i ) )
		end
	end
	checkEqual( success, true )
	
end

--------------------------------------------------------------------------------

WFCatVideoTimeTest.testBasic = function()

	local kNTSCFrameRate = WFCatVideoTime.kNTSCFrameRate
	
	checkEqual( WFCatVideoTime.formatSmpteVideoTime( 97.5, { 10, 1 } ), "01:37:05" )
	checkEqual( WFCatVideoTime.formatSmpteVideoTime( 120, kNTSCFrameRate ), "01;59;28" )
	
	local decoded = WFCatVideoTime.decodeUserVideoTime( "1;2", kNTSCFrameRate, "smpte" )
	checkEqual( decoded - 1.06773 < 0.001, true, "decode smtpe to decimal seconds" )
	local fmted = WFCatVideoTime.formatSmpteVideoTime( decoded, kNTSCFrameRate )
	checkEqual( fmted, "00;01;02" )

end

--------------------------------------------------------------------------------

return WFCatVideoTimeTest

