--[[----------------------------------------------------------------------------

WFSQLiteDatabaseUtils.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2017 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

local Rx = require 'Rx'
local Observable = Rx.Observable

	-- WF.core.os
local WFFileUtils = require 'WFFileUtils'

	-- WF.core.utils
local WFArguments = require 'WFArguments'

	-- WF.core.lua.threading
local WFWorkerThread = require "WFWorkerThread"

	-- WF.sqlite.database
local WFSQLiteDatabase = require 'WFSQLiteDatabase'
local WFSQLiteDatabaseSchema = require 'WFSQLiteDatabaseSchema'

--------------------------------------------------------------------------------

local check = assert( WFArguments.check )

--------------------------------------------------------------------------------

local function createInMemoryDbFromPath( path, schemaVersion )

	check( "+string", '+string' )

	local worker = WFWorkerThread( "WFSQLiteDatabaseUtilsWorker.lua" )

	return worker
			:request( "createInMemoryDbFromPath", path, schemaVersion )
			:map( 
				function( wfTransitableSQLiteDatabase )

					return WFSQLiteDatabase( 
							wfTransitableSQLiteDatabase, 
							WFSQLiteDatabaseSchema( schemaVersion) )

				end )

end

--------------------------------------------------------------------------------

local function onlineBackupDbToPathWithProgressStream( sourceDB, path )

	-- create the new db with the doc store schema
	local success, destDBOrErr = pcall( function()

		-- The schema will be replaced by the copied database.
		local emptySchema = WFSQLiteDatabaseSchema( '1.0' )
		return WFSQLiteDatabase( path, emptySchema )

	end )

	if not success then
		return Observable.throw( destDBOrErr )
	end

	local function canceledOrErrHandler()
		destDBOrErr:close()
		pcall( WFFileUtils.deleteFile, path )
	end

	return sourceDB:onlineBackupWithProgressStream( destDBOrErr, canceledOrErrHandler )
		:tapOnCompleted( function()
			destDBOrErr:close()
		end )

end

--==============================================================================

return {

	createInMemoryDbFromPath = createInMemoryDbFromPath,
	onlineBackupDbToPathWithProgressStream = onlineBackupDbToPathWithProgressStream,
}
